# Class import

from freshlybuiltimagebol.photo_se_nikali_awaj import PhotoAwaj
from freshlybuiltimagebol.text_bol_uthega import ShabdDhwani
from freshlybuiltimagebol.photo_se_text import PhotoShabd
from freshlybuiltimagebol.natural_photo_se_text import NaturalPhotoShabd
from freshlybuiltimagebol.OCR_Printed_Text import ImageProcess
from freshlybuiltimagebol.downloader import imagebol_model_downloader



from cv2 import fastNlMeansDenoisingColored
from cv2 import cvtColor
from cv2 import bitwise_not,threshold,getRotationMatrix2D
from cv2 import warpAffine,filter2D,imread
from cv2 import THRESH_BINARY,COLOR_BGR2GRAY,THRESH_OTSU
from cv2 import INTER_CUBIC,BORDER_REPLICATE,minAreaRect
from numpy import column_stack,array,where
from matplotlib.pyplot import imshow,xticks,yticks
from pytesseract import image_to_string,pytesseract 
from PIL import Image

class ImageProcess:
    '''this function is removing noise from the image'''
    def remove_noise(image):
        image = fastNlMeansDenoisingColored(image,None,20,10,7,21)
        return image

    '''this function is removing skewness.
    first, it calculate the angle and accordingly rotate image'''
    def remove_skew(image):
        in_gray = cvtColor(image, COLOR_BGR2GRAY)
        in_gray = bitwise_not(in_gray)
        thresh_pic = threshold(in_gray, 0, 255,THRESH_BINARY | THRESH_OTSU)[1]
        coords_x_y = column_stack(where(thresh_pic > 0))
        angle = minAreaRect(coords_x_y)[-1]
        if angle < -45:
            angle = -(90 + angle)
        else:
            angle = -angle
        (h, w) = image.shape[:2]
        center_of_pic = (w // 2, h // 2)
        M = getRotationMatrix2D(center_of_pic, angle, 1.0)
        image = warpAffine(image, M, (w, h),flags=INTER_CUBIC, borderMode=BORDER_REPLICATE)
        return image

    '''for removing blurness from the image,
    this function increase sharpness of the image.'''
    def sharpness_blur(image):
        sharpen_kernel = array([[-1,-1,-1], [-1,9,-1], [-1,-1,-1]])
        image = filter2D(image, -1, sharpen_kernel)
        return image

    '''using pytesseract, this function extracting text from the image.'''
    def to_text(image):
        try:
            pytesseract.tesseract_cmd = r"C:\Program Files\Tesseract-OCR\tesseract.exe"
            string_from_image = image_to_string(image,lang='eng')
        except Exception:
            pytesseract.tesseract_cmd = r"C:\Program Files(x86)\Tesseract-OCR\tesseract.exe"
            string_from_image = image_to_string(image,lang='eng')
        return string_from_image

    ##plot image in output
    def plot_image(image):
        imshow(image)
        xticks([])
        yticks([])




from requests import get
from os import mkdir,path,remove
from sys import stdout
from tqdm import tqdm
from time import sleep


class imagebol_model_downloader:
    
    status_code=0000
    
    def __init__(self,model_name,status_code=0):
        self.download_model(model_name,status_code)

    def download_model(self,model_name,status_code=0):
        available_models={"F_est":["frozen_east_text_detection","94.4MB"]}
        model_url= "https://raw.githubusercontent.com/FreshlyBuilt/freshlybuiltimagebol/master/freshlybuiltimagebol/models/"
        if model_name in available_models:
            model_name=available_models[model_name]
            if path.isfile("models/"+model_name[0]+".pb")==False:
                try:
                    mkdir("models")
                except:
                    try:
                        print('starting model download for the first time')
                        sleep(1)
                        print('download can take time depending upon your internet conection')
                        with open("models/"+model_name[0] +".pb", "wb") as f:
                            print(model_name[0]+" is of "+model_name[1])
                            choice=input("do you wish to download/ntype 'y':")
                            if (choice=='y'):
                                response = get(model_url+model_name[0]+".pb", stream=True)
                                    
                            else:
                                print('download canceled')
                            
                            try:
                                response.raise_for_status()   
                                    
                            except response.exceptions.HTTPError as errh:
                                print ("Http Error:",errh)
                                self.status_code=1004
                                return status_code
                            except response.exceptions.ConnectionError as errc:
                                print ("Error Connecting:",errc)
                                self.status_code=1005
                                return status_code
                            except response.exceptions.Timeout as errt:
                                print ("Timeout Error:",errt)
                                self.status_code=1006
                                return status_code
                            except response.exceptions.RequestException as err:
                                print ("OOps: Something Else",err)
                                self.status_code=1007
                                return status_code
                            
                            total_length = int(response.headers.get('content-length'))
                            if total_length is None:
                                f.write(response.content)
                            else:
                                chunk_size=1024
                                for data in tqdm(iterable = response.iter_content(chunk_size),total=total_length/chunk_size, unit = 'KB'):
                                    f.write(data)
                                tqdm.close()
                                f.close()
                        if total_length != 0 and tqdm.n != total_length:
                            print("ERROR, something went wrong")
                        print('model download successful')
                        self.status_code=1003
                        return status_code
                    except:
                        print('\ndownload interrupted')
                        try:
                            remove("models/"+model_name[0] +".pb")
                            self.status_code=1002
                            return status_code
                        except:
                            self.status_code=1002
                            return status_code
                            
            else:
                print('model already exist')
                self.status_code=1000
                return status_code
            
            
        else: 
            print("no reference found for "+model_name)
            self.status_code=1001
            return status_code

"""status_code_meanings"""
# 1000 - model already exist
# 1001 - model name incorrect
# 1002 - download interupted
# 1003 - successful download
# 1004 - http error
# 1005 - connection error
# 1006 - timeout error
# 1007 - miscellanious error
      

""" testing area"""
#model_name=input("model name: ")
#print(imagebol_model_downloader(model_name).status_code)           

    


from imutils.object_detection import non_max_suppression
from numpy import cos,sin,array
from pytesseract import image_to_string
from cv2 import resize,putText,FONT_HERSHEY_SIMPLEX,rectangle,imread,imshow
from cv2.dnn import readNet,blobFromImage
#from playsound import playsound
#from .text_bol_uthega import ShabdDhwani

'''
USE CASE
=====================
from freshlybuiltimagebol import NaturalPhotoShabd
img=cv2.imread(path_to_image)
out=NaturalPhotoShabd.text_pehchano(img)
cv2.imshow('output',out)
'''


class NaturalPhotoShabd:
    def result_vyakhya_kro(scores, geometry,min_confidence):
        (numRows, numCols) = scores.shape[2:4]
        rects = []
        confidences = []
        for y in range(0, numRows):
            scoresData = scores[0, 0, y]
            xData0 = geometry[0, 0, y]
            xData1 = geometry[0, 1, y]
            xData2 = geometry[0, 2, y]
            xData3 = geometry[0, 3, y]
            anglesData = geometry[0, 4, y]
            for x in range(0, numCols):
                if scoresData[x] < min_confidence:
                    continue
                (offsetX, offsetY) = (x * 4.0, y * 4.0)
                angle = anglesData[x]
                c = cos(angle)
                s = sin(angle)
                h = xData0[x] + xData2[x]
                w = xData1[x] + xData3[x]
                endX = int(offsetX + (c * xData1[x]) + (s * xData2[x]))
                endY = int(offsetY - (s * xData1[x]) + (c * xData2[x]))
                startX = int(endX - w)
                startY = int(endY - h)
                rects.append((startX, startY, endX, endY))
                confidences.append(scoresData[x])
        return (rects, confidences)
    
    def text_pehchano(image,min_confidence=0.85,width=320,height=320,padding=0.00):
        '''
        image=input image (type=numpy.ndarray)
        min_confidence= minimum confidence threshold to detect text from image (default=0.85)
        width=resizing width of image (default=320)
        height=resizing height of image (default=320)
        padding =
        '''
        east='freshlybuiltimagebol/models/frozen_east_text_detection.pb'
        orig = image.copy()
        (origH, origW) = image.shape[:2]
        (newW, newH) = (width, height)
        rW = origW / float(newW)
        rH = origH / float(newH)
        image = resize(image, (newW, newH))
        (H, W) = image.shape[:2]
        layerNames = [
            "feature_fusion/Conv_7/Sigmoid",
            "feature_fusion/concat_3"]
        net = readNet(east)
        blob = blobFromImage(image, 1.0, (W, H),
            (123.68, 116.78, 103.94), swapRB=True, crop=False)
        net.setInput(blob)
        (scores, geometry) = net.forward(layerNames)
        (rects, confidences) = NaturalPhotoShabd.result_vyakhya_kro(scores, geometry,min_confidence)
        boxes = non_max_suppression(array(rects), probs=confidences)
        results = []
        for (startX, startY, endX, endY) in boxes:
            startX = int(startX * rW)
            startY = int(startY * rH)
            endX = int(endX * rW)
            endY = int(endY * rH)
            dX = int((endX - startX) * padding)
            dY = int((endY - startY) * padding)
            startX = max(0, startX - dX)
            startY = max(0, startY - dY)
            endX = min(origW, endX + (dX * 2))
            endY = min(origH, endY + (dY * 2))
            roi = orig[startY:endY, startX:endX]
            config = ("-l eng --psm 7")
            text = image_to_string(roi, config=config)
            results.append(((startX, startY, endX, endY), text))
        results = sorted(results, key=lambda r:r[0][1])
        output = orig.copy()
        for ((startX, startY, endX, endY), text) in results:
            #print("OCR TEXT : ",text)
            text = "".join([c if ord(c) < 128 else "" for c in text]).strip()
            #ShabdDhwani.shabd_se_dhwani(text,'english',"out.mp3")
            #playsound('out.mp3')
            rectangle(output, (startX, startY), (endX, endY),
                (0, 0, 255), 1)
            putText(output, text, (startX, startY - 20),
                FONT_HERSHEY_SIMPLEX, 0.5, (255, 0, 0), 2)
        return output




from distutils.core import setup
from os import path

DESCRIPTION = """freshlybuiltimagebol is the library made to allow users to perform various
                 operations with text and images including translation to different languages,
                 generarting audio file from text,extracting text from image and many more."""

# The directory containing this file
this_directory = path.abspath(path.dirname(__file__))

# Import the README and use it as the long-description.
try:
    with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
        long_description = f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

setup(
  name = 'freshlybuiltimagebol',         
  packages = ['freshlybuiltimagebol'],   
  version = '0.0.2.5',     
  license='MIT',        
  description = 'Photo Bhi Bol Uthega',
  long_description=long_description,
  long_description_content_type="text/markdown",
  author = 'Vishal Sharma',                   
  author_email = 'vishalsharma.gbpecdelhi@gmail.com',      
  url = 'https://github.com/FreshlyBuilt/freshlybuiltimagebol',   
  download_url = 'https://github.com/FreshlyBuilt/freshlybuiltimagebol/archive/v0.0.2.5.tar.gz',  
  keywords = ['Image', 'Audio', 'Text'],   
  install_requires=[            
          'hyper',
          'googletrans',
          'gTTS',
          'Pillow',
          'pytesseract',
          'opencv-python',
          'numpy',
          'matplotlib',
          'imutils'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3',      
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)
